package jvm_error;

import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;

/**
 * A ObjectTransfer may be used for drag and drop operations within the same instance of
 * an application.
 */

public final class LocalObjectTransfer extends Transfer {

	private static LocalObjectTransfer instance = new LocalObjectTransfer();

	/**
	 * Returns the single instance of ObjectTransfer.
	 * 
	 * @return instance
	 */

	public static LocalObjectTransfer getInstance() {
		return instance;
	}

	private static final String TYPE_NAME = LocalObjectTransfer.class.getName();
	private static final int TYPE_ID = registerType(TYPE_NAME);

	private Object draggedObject;

	private LocalObjectTransfer() {
		// hide
	}

	@Override
	public void javaToNative(Object object, TransferData transferData) {
		if (!isSupportedType(transferData)) {
			DND.error(DND.ERROR_INVALID_DATA);
		}
		this.draggedObject = object;
	}

	@Override
	public Object nativeToJava(TransferData transferData) {
		Object result = this.draggedObject;
		this.draggedObject = null;
		return result;
	}

	@Override
	protected String[] getTypeNames() {
		return new String[] { TYPE_NAME };
	}

	@Override
	protected int[] getTypeIds() {
		return new int[] { TYPE_ID };
	}

	@Override
	protected boolean validate(Object object) {
		return object != null;
	}

	@Override
	public TransferData[] getSupportedTypes() {
		int[] types = getTypeIds();
		TransferData[] data = new TransferData[types.length];
		for (int i = 0; i < types.length; i++) {
			data[i] = new TransferData();
			data[i].type = types[i];
		}
		return data;
	}

	@Override
	public boolean isSupportedType(TransferData transferData) {
		boolean result = false;
		if (transferData != null) {
			int[] types = getTypeIds();
			for (int i = 0; i < types.length; i++) {
				if (transferData.type == types[i]) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

}